﻿% Migrate Forms from any Zim versions to Zim 9.50 format.
% Celso Bressan - August/2023.
procedure GUIMigrate() local(vlHexa, vlTemp)
    % Do some initial checking.
    change all FormFields where WdgCSS is $null let WdgCSS = ""
    change all FormFields where WdgCSS in ("0", "{}") let WdgCSS = ""
    
    % Remove the crappy BMP image.
    change all FormFields where Default = "^*.BMP" let Default = ""
    change all FormFields where WdgOnImage = "^*.BMP" let WdgOnImage = ""
    change all FormFields where WdgOffImage = "^*.BMP" let WdgOffImage = ""
    change all FormFields where WdgNullImage = "^*.BMP" let WdgNullImage = "" 

    % Migrate FT to WdgType. FT will no longer be in use.
    change all FormFields where FT > "" and WdgType > 0 let FT = ""
    change all FormFields where FT > "" and {WdgType, 0} = 0 let \
        WdgType = {30 where FT = "0", 31 where FT = "1", 20 where FT = "A", 04 where FT = "B", 08 where FT = "C", \
            12 where FT = "D", 17 where FT = "E", 06 where FT = "F", 11 where FT = "G", 23 where FT = "H", 10 where FT = "I", \
            22 where FT = "J", 25 where FT = "K", 07 where FT = "L", 05 where FT = "M", 19 where FT = "N", 09 where FT = "O", \
            01 where FT = "P", 29 where FT = "Q", 21 where FT = "R", 13 where FT = "S", 03 where FT = "T", 14 where FT = "U", \
            02 where FT = "V", 24 where FT = "W", 28 where FT = "X", 26 where FT = "Y", 27 where FT = "Z", 07} \
        FT = ""
        
    % Migrate the old button CSS format in Default to the proper WdgCSS field.
    change all FormFields where Default = "B^"? let \
        WdgCSS = $concat('{"BCGradient":["', $substring(Default, 3, 2), '","', $substring(Default, 5, 2), '","#', $substring(Default, 7, 6), '","#', $substring(Default, 13, 6), '"],', \
            ' "FCGradient":["', $substring(Default, 22, 2), '","', $substring(Default, 24, 2), '","#', $substring(Default, 26, 6), '","#', $substring(Default, 32, 6), '"],', \
            ' "Border":[', $trim($tonumber($substring(Default, 41, 2), 0)), ',', $trim($tonumber($substring(Default, 43, 2), 0)), ',"#', $substring(Default, 45, 6), '"]}') \
        Default = ""

    % Migrate the Microsoft Sans Serif font.
    change all FormFields where WdgFont = "MS Sans Serif" let WdgFont = "Microsoft Sans Serif"

    % Check the consistency of the color information.
    find FFS -> sFFSTotal
    list all sFFSTotal where (let $page = {WdgBCRed, -1000} + {WdgBCGreen, -1000} + {WdgBCBlue, -1000}) between -2999 and -1 or $page > 765 \
        format $concat("Background color inconsistency in Form ", $trim(DirName), ".", $trim(FormName), ".") -> sFFSWrong
    find sFFSTotal minus sFFSWrong -> sFFSTotal
    list all sFFSTotal where (let $page = {WdgFCRed, -1000} + {WdgFCGreen, -1000} + {WdgFCBlue, -1000}) between -2999 and -1 or $page > 765 \
        format $concat("Foreground color inconsistency in Form ", $trim(DirName), ".", $trim(FormName), ".") -> sFFSWrong
    find sFFSTotal minus sFFSWrong -> sFFSTotal

    % Make BCColor and FCColor in WdgCSS.
    let vlHexa = "0123456789ABCDEF"
    find sFFSTotal where WdgBCRed is not $null -> sFFSBCColor
    change all sFFSBCColor let WdgCSS = $updateJSON(WdgCSS, "", \
            ((let $page = $truncate(WdgBCRed / 16)), (let $located = WdgBCRed - $page * 16), \
            (let vlTemp = $concat('"BCColor":"#', $substring(vlHexa, $page + 1, 1), $substring(vlHexa, $located + 1, 1))), \
            (let $page = $truncate(WdgBCGreen / 16)), (let $located = WdgBCGreen - $page * 16), \
            (let vlTemp = $concat(vlTemp, $substring(vlHexa, $page + 1, 1), $substring(vlHexa, $located + 1, 1))), \
            (let $page = $truncate(WdgBCBlue / 16)), (let $located = WdgBCBlue - $page * 16), \
            (let vlTemp = $concat(vlTemp, $substring(vlHexa, $page + 1, 1), $substring(vlHexa, $located + 1, 1), '"')))) \
        WdgBCRed = $null \
        WdgBCGreen = $null \
        WdgBCBlue = $null
    find sFFSTotal where WdgFCRed is not $null -> sFFSFCColor
    change all sFFSFCColor let WdgCSS = $updateJSON(WdgCSS, "", \
            ((let $page = $truncate(WdgFCRed / 16)), (let $located = WdgFCRed - $page * 16), \
            (let vlTemp = $concat('"FCColor":"#', $substring(vlHexa, $page + 1, 1), $substring(vlHexa, $located + 1, 1))), \
            (let $page = $truncate(WdgFCGreen / 16)), (let $located = WdgFCGreen - $page * 16), \
            (let vlTemp = $concat(vlTemp, $substring(vlHexa, $page + 1, 1), $substring(vlHexa, $located + 1, 1))), \
            (let $page = $truncate(WdgFCBlue / 16)), (let $located = WdgFCBlue - $page * 16), \
            (let vlTemp = $concat(vlTemp, $substring(vlHexa, $page + 1, 1), $substring(vlHexa, $located + 1, 1), '"')))) \
        WdgFCRed = $null \
        WdgFCGreen = $null \
        WdgFCBlue = $null
        
    % Process BC and FC with no widget color assigned, not inherited color and not a Windows color.
    let vlHexa = "#000000#000080#008000#008080#FF0000#FF00FF#800000#C0C0C0#808080#0000FF#00FF00#00FFFF#800080#FF00FF#FFFF00#FFFFFF"
    find sFFSTotal minus sFFSBCColor -> sFFSProcess
    change all sFFSProcess where BC between 1 and 16 \
        let WdgCSS = $updatejson(WdgCSS, "", $concat('"BCColor":"', $substring(vlHexa, (BC - 1) * 7 + 1, 7), '"'))
    find sFFSTotal minus sFFSFCColor -> sFFSProcess
    change all sFFSProcess where FC between 1 and 16 \
        let WdgCSS = $updatejson(WdgCSS, "", $concat('"FCColor":"', $substring(vlHexa, (FC - 1) * 7 + 1, 7), '"'))

    % Correct the borders for buttons and frames.
    change all FormFields where WdgType = 6 or WdgType = 9 \
        let WdgCSS = $updatejson(WdgCSS, "", '"Border":[1,0,"#000000"]')

    % Recreate everything.
    find Forms
    if $setcount > 0     
        zomrecreate +t Forms
    endif
    find Displays
    if $setcount > 0     
        zomrecreate +t Displays
    endif
    find Menus
    if $setcount > 0     
        zomrecreate +t Menus
    endif
    find Windows
    if $setcount > 0     
        zomrecreate +t Windows
    endif
endprocedure