﻿using Microsoft.Win32;
using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;

namespace ZimConnection
{
    public class ZimAPI
    {
        public static int hConn;

        public const int TOP = -2147483647;
        public const int BOTTOM = 2147483647;
        public const int UTF8 = 0;
        public const int ANSI = 1;
        public const int NO_ERROR = 0;
        public const int API_NO_ERROR = 0;
        public const string ZIM_REGISTRY = "Software\\Zim\\9.50";
        public const string ZIM_NAME = "ZIM";

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIOpen(ref int HConn, string zimConnectString, string ClientType);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIClose(ref int HConn);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern IntPtr APIExecuteCall(int HConn, int NumIHV, int NumOHV, string pCommand);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIExecute(int HConn, string pCommand);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APICopyFromServer(int hConn, string pSource, string pDest, string pFormat);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APICopyToServer(int hConn, string pSource, string pDest, string pFormat);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern string APILastErrorMsg(int hConn);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APILastErrorCode(int hConn);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APICreateCursor(int hConn);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIExecuteCursor(int hConn, string pCommand, int SetNumber);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIRowCount(int hConn, int SetNumber);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIRowNumber(int hConn, int SetNumber);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern IntPtr APIFetchOne(int hConn, int SetNumber);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APICloseCursor(int hConn, int SetNumber);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern IntPtr APIValueOf(int hConn, int SetNumber, int Index, string pKey);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern IntPtr APIGetName(int hConn, int SetNumber, int Index);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIFieldCount(int hConn, int SetNumber);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIBeginTrans(int hConn);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIEndTrans(int hConn);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIQuitTrans(int hConn);

        [DllImport("zimapi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern int APIScrollSet(int hConn, int SetNumber, int Index);

        public class ZimCursor
        {
            public int SetNumber;

            public ZimCursor()
            {
                SetNumber = APICreateCursor(hConn);
            }

            public ZimCursor(string pCommand)
            {
                SetNumber = APICreateCursor(hConn);
                APIExecuteCursor(hConn, pCommand, SetNumber);
            }

            public int Execute(string pCommand)
            {
                ZimAPI.State = APIExecuteCursor(hConn, pCommand, SetNumber);
                return ZimAPI.State;
            }

            public int RowCount()
            {
                return APIRowCount(hConn, SetNumber);
            }

            public int RowNumber()
            {
                return APIRowNumber(hConn, SetNumber);
            }

            public string FetchOne()
            {
                var row = APIFetchOne(hConn, SetNumber);
                string strResult;

                if (row == null)
                    return null;
                strResult = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(row);
                return strResult;
            }

            public int Close()
            {
                return APICloseCursor(hConn, SetNumber);
            }

            private int GetUtf8StringLength(IntPtr ptr)
            {
                int length = 0;
                byte byteValue;

                if (ptr != IntPtr.Zero)
                {
                    while ((byteValue = Marshal.ReadByte(ptr, length)) != 0)
                    {
                        length++;
                    }
                }
                return length;
            }

            public string ValueOf(string pKey)
            {
                var result = APIValueOf(hConn, SetNumber, -1, pKey);

                return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(result);
            }

            public string ValueOf(string pKey, int Code)
            {
                var result = APIValueOf(hConn, SetNumber, -1, pKey);

                if (Code == UTF8)
                {
                    int length = GetUtf8StringLength(result);
                    byte[] byteArray = new byte[length];
                    if (length > 0)
                        Marshal.Copy(result, byteArray, 0, length);
                    return Encoding.UTF8.GetString(byteArray);
                }
                else
                    return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(result);
            }

            public string ValueOf(int Index)
            {
                var result = APIValueOf(hConn, SetNumber, Index, null);

                return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(result);
            }

            public string ValueOf(int Index, int Code)
            {
                var result = APIValueOf(hConn, SetNumber, Index, null);

                if (Code == UTF8)
                {
                    int length = GetUtf8StringLength(result);
                    byte[] byteArray = new byte[length];
                    if (length > 0)
                        Marshal.Copy(result, byteArray, 0, length);
                    return Encoding.UTF8.GetString(byteArray);
                }
                else
                    return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(result);
            }

            public int FieldCount()
            {
                return APIFieldCount(hConn, SetNumber);
            }

            public string GetName(int Index)
            {
                var result = APIGetName(hConn, SetNumber, Index);
                return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(result);
            }

            public int Scroll(int Index)
            {
                ZimAPI.State = APIScrollSet(hConn, SetNumber, Index);

                return ZimAPI.State;
            }
        }

        public static int State
        {
            get;
            internal set;
        }

        public ZimAPI(string dataBase, string hostName, string portNum, string userName, string passwd)
        {
            Connect(dataBase, hostName, portNum, userName, passwd);
        }

        public ZimAPI()
        {
        }

        public int Connect(string dataBase, string hostName, string portNum, string userName, string passwd)
        {
            RegistryKey key;
            string ZimPath;
            string zimString;

            if ((key = Registry.LocalMachine.OpenSubKey(ZimAPI.ZIM_REGISTRY)) != null)
            {
                ZimPath = (string)key.GetValue(ZimAPI.ZIM_NAME);
                if (ZimPath.EndsWith("\\") == false)
                    ZimPath += "\\";
            }
            else
            {
                ZimAPI.State = 9903;
                return ZimAPI.State;
            }

            Directory.SetCurrentDirectory(ZimPath);
            zimString = "Port=" + portNum + ";Host=" + hostName + ";Password=" + passwd + ";User=" + userName + ";dbname=" + dataBase + ";";
            State = APIOpen(ref hConn, zimString, "CSHARP");

            return ZimAPI.State;
        }

        public int Close()
        {
            ZimAPI.State = APIClose(ref hConn);

            return ZimAPI.State;
        }

        public string[] CallProc(string pCommand)
        {
            string[]    TheParms = pCommand.Split(new char[] {'(', ',', ')'});
            string[]    BackParms;
            string[]    FinalParms = new string[10];
            string[]    TheXParms = pCommand.Split('@');
            int         i;
            int         InHV = 0;
            int         OutHV = 0;
            int         j;
            int         k;

            for (i = 1; i < TheParms.Length; i++)
            {
                TheParms[i] = TheParms[i].TrimStart();
            }
            for (i = 1; i < TheParms.Length; i++)
            {
                if (TheParms[i].StartsWith("!"))
                    InHV++;
                else
                    if (TheParms[i].StartsWith("@"))
                        OutHV++;
            }

            var result = APIExecuteCall(hConn, InHV, OutHV, pCommand);
            var strResult = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(result);
            BackParms = strResult.Split('^');

            j = 0;
            k = 1;
            for (i = 0; i < TheParms.Length; i++)
            {
                if (TheParms[i].StartsWith("@"))
                    FinalParms[k++] = BackParms[j++];
            }
            for (i = 1; i < k; i++)
            {
                if (FinalParms[i] != null)
                    FinalParms[i] = Regex.Replace(FinalParms[i], "^\"|\"$", "");
            }

            return FinalParms;
        }

        public int Execute(string pCommand)
        {
            ZimAPI.State = APIExecute(hConn, pCommand);

            return ZimAPI.State;
        }

        public int GetFile(string pSource, string pDest, string pFormat)
        {
            ZimAPI.State = APICopyFromServer(hConn, pSource, pDest, pFormat);

            return ZimAPI.State;
        }

        public int PutFile(string pSource, string pDest, string pFormat)
        {
            ZimAPI.State = APICopyToServer(hConn, pSource, pDest, pFormat);

            return ZimAPI.State;
        }

        public string ErrorMessage()
        {
            return APILastErrorMsg(hConn);
        }

        public int ErrorCode()
        {
            ZimAPI.State = APILastErrorCode(hConn);

            return ZimAPI.State;
        }

        public ZimCursor Cursor()
        {
            return new ZimCursor();
        }

        public ZimCursor Cursor(string pCommand)
        {
            return new ZimCursor(pCommand);
        }

        public int Transaction()
        {
            ZimAPI.State = APIBeginTrans(hConn);

            return ZimAPI.State;
        }

        public int Commit()
        {
            ZimAPI.State = APIEndTrans(hConn);

            return ZimAPI.State;
        }

        public int Rollback()
        {
            ZimAPI.State = APIQuitTrans(hConn);

            return ZimAPI.State;
        }
   }
}

